@@ -6,16 +6,17 @@ structs, better handling for strings and arrays, struct inheritance, and
 convenient syntax for instantiating and using your custom structs.
 """
 import sys
+import codecs
 import struct
 import socket
 import logging
 import warnings
 import traceback
-from time import mktime
 from copy import deepcopy
 from select import select
 from warnings import warn
 from types import NoneType
+from time import mktime, time
 from datetime import datetime
 from StringIO import StringIO
 from logging.handlers import RotatingFileHandler
@@ -24,10 +25,10 @@ from SocketServer import TCPServer, UDPServer, StreamRequestHandler, DatagramReq
 
 __all__ = ["CError", "CWarning",
            "CType", "CStruct", "CStructType", "CArray",
-           "CChar", "CUChar", "CShort", "CUShort", "CInt", "CUInt", "CLong", "CULong", "CFloat", "CDouble", "CString",
+           "CChar", "CUChar", "CShort", "CUShort", "CInt", "CUInt", "CLong", "CULong", "CFloat", "CDouble", "CString", "CUnicode",
            "Parser", "Logger", "ProtHandler", "TCPHandler", "UDPHandler", "LoggingTCPServer", "LoggingUDPServer",
            "underscorize", "hexdump",
-           "BYTE_ORDER", "DEFAULT_TIMEOUT"]
+           "BYTE_ORDER", "AUTOSIZED"]
 
 class CError(ValueError):
     """the only exception class raised directly by protlib"""
@@ -35,11 +36,11 @@ class CError(ValueError):
 class CWarning(UserWarning):
     """the only warning class directly used by protlib (except DeprecationWarning)"""
 
-__version_info__ = (1, 2, 0, "final", 0)
+__version_info__ = (1, 3, 0, "final", 0)
 __version__ = "{0}.{1}.{2}".format(*__version_info__)
 
 BYTE_ORDER = "!"
-DEFAULT_TIMEOUT = 5     # seconds
+AUTOSIZED = "AUTO SIZING"
 
 def _get_default(val):
     return val() if hasattr(val, "__call__") else deepcopy(val)
@@ -47,6 +48,18 @@ def _get_default(val):
 def _fileize(x):
     return StringIO(x) if isinstance(x, basestring) else x
 
+def _no_codec(name):
+    try:
+        codecs.lookup(name)
+    except:
+        return True
+
+def _is_open(sock):
+    try:
+        return isinstance(sock.fileno(), int)
+    except:
+        return False
+
 def _inherit_docstrings(klass):
     for name,field in klass.__dict__.items():
         parent = getattr(klass.__bases__[0], name, None)
@@ -54,6 +67,17 @@ def _inherit_docstrings(klass):
             field.__doc__ = parent.__doc__
     return klass
 
+def _read_until_null(f):
+    s = ""
+    f = _fileize(f)
+    while True:
+        c = f.read(1)
+        if c in ["","\0"]:
+            break
+        else:
+            s += c      # shlemiel the painter
+    return s
+
 class _chained:
     def __init__(self, s, f):
         self.s, self.f = s, f
@@ -65,7 +89,7 @@ class _chained:
             data = self.s[pos : pos + n]
         else:
             data = self.f.read(n)
-        self.has_read += data       # shlemeil the painter
+        self.has_read += data
         return data
 
 class CType(object):
@@ -90,9 +114,9 @@ class CType(object):
     
     def __init__(self, **settings):
         """
-        CTypes have three optional keyword arguments:
+        CTypes have five optional keyword arguments:
         
-        length -- required for CString and CArray,
+        length -- required for CString, CUnicode, and CArray,
                   invalid for everything else
         
         always -- Use this to set a constant value for a field. You won't
@@ -102,9 +126,15 @@ class CType(object):
         default -- Like the always parameter, except that no warnings are
                    raised when a different value is assigned, parsed, or 
                    serialized. Also, unlike an always parameter, a default 
-                   parameter may be  either a value or a callable object.
+                   parameter may be either a value or a callable object.
+        
+        encoding -- required for CUnicode, invalid for everything else
+        
+        enc_errors -- Only valid (but not required) for CUnicode, see the
+                      unicode "errors" parameter for details.  This defaults
+                      to "strict" but may be set to "ignore", "replace", etc
         """
-        self.always = self.default = self.length = None
+        self.always = self.default = self.length = self.encoding = self.enc_errors = None
         extra = [name for name,val in settings.items() if not hasattr(self, name)]
         if extra:
             warn("{0} settings do not include {1}".format(self.__class__.__name__, ", ".join(extra)), CWarning)
@@ -112,14 +142,25 @@ class CType(object):
         self.__dict__.update(settings)
         if self.__class__ is CType:
             raise CError("CType may not be directly instantiated; use a subclass such as CInt, CString, etc")
-        if self.length is not None and not isinstance(self, (CString, CArray)):
+        if self.length is not None and not isinstance(self, (CString, CUnicode, CArray)):
             warn("length has no meaning for {0} objects".format(self.__class__.__name__), CWarning)
-        if isinstance(self, (CString, CArray)):
+        if isinstance(self, (CString, CUnicode, CArray)):
             if not isinstance(self.length, (int, basestring)):
                 raise CError("{0} objects require a string or integer length attribute".format(self.__class__.__name__))
             elif isinstance(self.length, int) and self.length < 0:
                 raise CError("length integer value must be positive")
         
+        if isinstance(self, CUnicode):
+            self.enc_errors = "strict" if self.enc_errors is None else self.enc_errors
+            if not self.encoding:
+                raise CError("CUnicode objects require an encoding parameter")
+            elif _no_codec(self.encoding):
+                raise CError("no codec exists for specified encoding {0!r}".format(self.encoding))
+        else:
+            for attr in ["encoding","enc_errors"]:
+                if getattr(self, attr):
+                    raise CError("{0} parameter is not valid for {0} objects".format(attr, self.__class__.__name__))
+        
         self.instances.append(self)
     
     @property
@@ -143,29 +184,37 @@ class CType(object):
         if cstruct is None:
             raise CError("cstruct not provided to resolve variable-length field with length attribute {0!r}".format(self.length))
         
-        length = getattr(cstruct, self.length, None)
-        if isinstance(length, (CType, NoneType)):
-            raise CError("{0}.{1} not set and is needed for use as a length attribute".format(cstruct.__class__.__name__, self.length))
-        elif not isinstance(length, int):
-            raise CError("{0}.{1} must be an integer for use as a length attribute".format(cstruct.__class__.__name__, self.length))
-        elif length < 0:
-            raise CError("length field {0}.{1} may not be negative".format(cstruct.__class__.__name__, self.length))
-        return length
+        if self.length is AUTOSIZED:
+            field_name = dict((v,k) for k,v in cstruct.__dict__.items() if getattr(v, "__hash__", None))[self]
+            field_val = getattr(cstruct, field_name)
+            if not isinstance(field_val, basestring):
+                raise CError("{0}.{1} is set to a non-string value: {2!r}".format(cstruct.__class__.__name__, field_name, field_val))
+            return len(field_val)
+        else:
+            length = getattr(cstruct, self.length, None)
+            if isinstance(length, (CType, NoneType)):
+                raise CError("{0}.{1} not set and is needed for use as a length attribute".format(cstruct.__class__.__name__, self.length))
+            elif not isinstance(length, int):
+                raise CError("{0}.{1} must be an integer for use as a length attribute".format(cstruct.__class__.__name__, self.length))
+            elif length < 0:
+                raise CError("length field {0}.{1} may not be negative".format(cstruct.__class__.__name__, self.length))
+            return length
     
     def struct_format(self, cstruct=None):
         """the format string used to represent this CType in the struct module"""
         return {
-            CChar:   "b",
-            CUChar:  "B",
-            CShort:  "h",
-            CUShort: "H",
-            CInt:    "i",
-            CUInt:   "I",
-            CLong:   "q",
-            CULong:  "Q",
-            CFloat:  "f",
-            CDouble: "d",
-            CString: "{0}s".format( self.real_length(cstruct) )
+            CChar:    "b",
+            CUChar:   "B",
+            CShort:   "h",
+            CUShort:  "H",
+            CInt:     "i",
+            CUInt:    "I",
+            CLong:    "q",
+            CULong:   "Q",
+            CFloat:   "f",
+            CDouble:  "d",
+            CString:  "{0}s".format( self.real_length(cstruct) ),
+            CUnicode: "{0}s".format( self.real_length(cstruct) )
         }[self.__class__]
     
     def sizeof(self, cstruct=None):
@@ -180,7 +229,7 @@ class CType(object):
         
         CString parsing returns the string up to the first null byte, e.g.
         
-        CString(length=10).parse("foo\x00barbaz") -> "foo"
+        CString(length=10).parse("foo\0barbaz") -> "foo"
         """
         buf = _fileize(f).read( self.sizeof(cstruct) )
         if len(buf) < self.sizeof(cstruct):
@@ -214,7 +263,7 @@ class CType(object):
                 return struct.pack(BYTE_ORDER + self.struct_format(cstruct), val)
         except:
             exc_class, exc, tb = sys.exc_info()
-            cerror = CError("{0!r} is not serializable as a {1}: {2}".format(val, self.__class__.__name__, exc.args[0]))
+            cerror = CError("{0!r} is not serializable as a {1}: {2}".format(val, self.__class__.__name__, exc))
             raise CError, cerror, tb
 
 class CChar(CType): pass
@@ -231,12 +280,47 @@ class CDouble(CType): pass
 @_inherit_docstrings
 class CString(CType):
     def parse(self, f, cstruct=None):
-        return CType.parse(self, f, cstruct).split("\x00")[0]
+        if self.length is AUTOSIZED:
+            return _read_until_null(f)
+        else:
+            return CType.parse(self, f, cstruct).split("\0")[0]
     
     def serialize(self, val, cstruct=None):
-        if len(val) > self.real_length(cstruct):
-            warn("CString has length {0} and was told to serialize a string of length {1}".format(self.real_length(cstruct), len(val)), CWarning)
-        return CType.serialize(self, val, cstruct)
+        if self.length is AUTOSIZED:
+            return val.rstrip("\0") + "\0"
+        else:
+            if len(val) > self.real_length(cstruct):
+                warn("CString has length {0} and was told to serialize a string of length {1}".format(self.real_length(cstruct), len(val)), CWarning)
+            return CType.serialize(self, val, cstruct)
+
+@_inherit_docstrings
+class CUnicode(CType):
+    def parse(self, f, cstruct=None):
+        if self.length is not AUTOSIZED:
+            return CType.parse(self, f, cstruct).split("\0")[0].decode(self.encoding, self.enc_errors)
+        else:
+            s = _read_until_null(f)
+            try:
+                return s.decode(self.encoding, self.enc_errors)
+            except:
+                exc_class, exc, tb = sys.exc_info()
+                cerror = CError("unicode error parsing {0!r}: {1}".format(s, exc))
+                raise CError, cerror, tb
+    
+    def serialize(self, val, cstruct=None):
+        try:
+            encoded = _get_converter(self)(val).encode(self.encoding, self.enc_errors)
+        except:
+            exc_class, exc, tb = sys.exc_info()
+            cerror = CError("unicode error serializing {0!r}: {1}".format(val, exc))
+            raise CError, cerror, tb
+        
+        if self.length is AUTOSIZED:
+            return encoded + "\0"
+        else:
+            if len(encoded) > self.real_length(cstruct):
+                warn("CUnicode value has length {0} and was told to serialize an encoded string of length {1} {2!r}".format(self.real_length(cstruct), len(encoded), encoded), CWarning)
+            return CType.serialize(self, encoded, cstruct)
 
 @_inherit_docstrings
 class CArray(CType):
@@ -270,7 +354,7 @@ class CArray(CType):
         
         for param in ["default","always"]:
             if isinstance(length, int) and param not in params and getattr(self.ctype, param) is not None:
-                params[param] = [_get_default(getattr(self.ctype, param)) for i in range(length)]
+                params[param] = [_get_default(getattr(self.ctype, param)) for i in xrange(length)]
         
         CType.__init__(self, length=length, **params)
         
@@ -281,7 +365,7 @@ class CArray(CType):
                     self.serialize( _get_converter(self)(value) )
                 except:
                     exc_class, exc, tb = sys.exc_info()
-                    cerror = CError("{0!r} is not a valid {1} CArray value: {2}".format(value, param, exc.args[0]))
+                    cerror = CError("{0!r} is not a valid {1} CArray value: {2}".format(value, param, exc))
                     raise CError, cerror, tb
     
     def struct_format(self, cstruct=None):
@@ -289,7 +373,7 @@ class CArray(CType):
     
     def parse(self, f, cstruct=None):
         f = _fileize(f)
-        return [self.ctype.parse(f, cstruct) for i in range(self.real_length(cstruct))]
+        return [self.ctype.parse(f, cstruct) for i in xrange(self.real_length(cstruct))]
     
     def serialize(self, xs, cstruct=None):
         length = self.real_length(cstruct)
@@ -300,7 +384,7 @@ class CArray(CType):
             if self.maybe is not None or self.ctype.maybe is not None:
                 default = self.maybe or []
                 if self.ctype.maybe is not None:
-                    default += [self.ctype.maybe for i in range(length - len(default))]
+                    default += [self.ctype.maybe for i in xrange(length - len(default))]
                 xs = xs + default[len(xs):]     # avoid += to not mutate the original list
             if len(xs) < length:
                 raise CError("CArray has length {0} and was only given {1} elements".format(length, len(xs)))
@@ -360,7 +444,7 @@ class CStructType(CType):
                         for maybe in [ctype.default, ctype.always, ctype.ctype.default, ctype.ctype.always]:
                             if maybe is not None:
                                 val = []
-                elif isinstance(ctype, CString) and ctype.real_length(inst) == 0:
+                elif isinstance(ctype, (CString, CUnicode)) and ctype.real_length(inst) == 0:
                     val = ""
             if val is None:
                 raise CError(name + " not set")
@@ -399,12 +483,14 @@ class CStruct(CType):
         for name,ctype in fields:
             if ctype.maybe is not None:
                 setattr(self, name, _get_default(ctype.maybe))
-            elif isinstance(ctype, CString) and isinstance(ctype.length, basestring) \
-                    and not isinstance(getattr(self, ctype.length), CType) and ctype.real_length(self) == 0:
+            elif isinstance(ctype, (CString, CUnicode)) and isinstance(ctype.length, basestring) \
+                    and ctype.length is not AUTOSIZED \
+                    and not isinstance(getattr(self, ctype.length), CType) \
+                    and ctype.real_length(self) == 0:
                 setattr(self, name, "")
             elif isinstance(ctype, CArray) and isinstance(ctype.length, basestring) and ctype.ctype.maybe is not None \
                     and not isinstance(getattr(self, ctype.length), CType):
-                setattr(self, name, [_get_default(ctype.ctype.maybe) for i in range(ctype.real_length(self))])
+                setattr(self, name, [_get_default(ctype.ctype.maybe) for i in xrange(ctype.real_length(self))])
             
             if name in values:
                 setattr(self, name, values[name])   # set after setting default values to detect invalid defaults
@@ -445,7 +531,7 @@ class CStruct(CType):
                 fields.sort(key = lambda pair: CType.instances.index(pair[1]))
                 
                 positions = [(CType.instances.index(ctype),name,ctype) for name,ctype in fields]
-                for i in range(1, len(positions)):
+                for i in xrange(1, len(positions)):
                     if positions[i][0] == positions[i-1][0]:
                         warn("{0} and {1} were declared with the same {2} object; the order of such fields is undefined".format(positions[i-1][1], positions[i][1], positions[i][2].__class__.__name__), CWarning)
                         break
@@ -572,14 +658,14 @@ class CStruct(CType):
                 value = _get_converter(field)(value)
             except:
                 exc_class, exc, tb = sys.exc_info()
-                cerror = CError("Conversion error: you provided the {0} value {1!r} to the {2} field {3}.{4}: {5}".format(value.__class__.__name__, value, field.__class__.__name__, self.__class__.__name__, name, exc.args[0]))
+                cerror = CError("Conversion error: you provided the {0} value {1!r} to the {2} field {3}.{4}: {5}".format(value.__class__.__name__, value, field.__class__.__name__, self.__class__.__name__, name, exc))
                 raise CError, cerror, tb
             
             try:
                 field.serialize(value, cstruct=self)
             except:
                 exc_class, exc, tb = sys.exc_info()
-                cerror = CError("{0!r} is an invalid value for the {1} field {2}.{3}: {4}".format(value, field.__class__.__name__, self.__class__.__name__, name, exc.args[0]))
+                cerror = CError("{0!r} is an invalid value for the {1} field {2}.{3}: {4}".format(value, field.__class__.__name__, self.__class__.__name__, name, exc))
                 raise CError, cerror, tb
             
             if field.always is not None and value != field.always:
@@ -588,11 +674,11 @@ class CStruct(CType):
         object.__setattr__(self, name, value)
 
 _converters = {
-    CFloat:  float,
-    CDouble: float,
-    CString: str,
-    CChar:   lambda c: ord(c) if isinstance(c, basestring) else int(c),
-    CUChar:  lambda c: ord(c) if isinstance(c, basestring) else int(c)
+    CFloat:   float,
+    CDouble:  float,
+    CString:  str,
+    CChar:    lambda c: ord(c) if isinstance(c, basestring) else int(c),
+    CUChar:   lambda c: ord(c) if isinstance(c, basestring) else int(c)
 }
 def _to_int(x):
     if isinstance(x, float) and x != int(x):
@@ -603,6 +689,8 @@ _converters.update((ctype, _to_int) for ctype in [CShort, CUShort, CInt, CUInt,
 def _get_converter(field):
     if isinstance(field, CStructType):
         return lambda cstruct: cstruct
+    elif isinstance(field, CUnicode):
+        return lambda x: x if isinstance(x, unicode) else unicode(str(x), field.encoding, field.enc_errors)
     elif isinstance(field, CArray):
         def converter(xs):
             conv = _get_converter(field.ctype)
@@ -655,7 +743,7 @@ def hexdump(data):
     """
     hexed = [hex(ord(char))[2:].rjust(2,"0") for char in data]
     lines = ["     0  1  2  3  4  5  6  7"]
-    for i in range(0, len(hexed), 8):
+    for i in xrange(0, len(hexed), 8):
         lines.append("%3i  " % i + " ".join(hexed[i:i+8]))
     return "\n".join(lines)
 
@@ -825,7 +913,8 @@ class Logger(object):
             
             if not logger.handlers:
                 try:
-                    handler = RotatingFileHandler(logger.name+"_log", maxBytes=self.MAX_BYTES, backupCount=self.BACKUP_COUNT)
+                    delay = sys.version_info[:3] >= (2, 6, 2)
+                    handler = RotatingFileHandler(logger.name+"_log", maxBytes=self.MAX_BYTES, backupCount=self.BACKUP_COUNT, delay=delay)
                     handler.setFormatter(_formatter)
                     handler.setLevel(rfh_level)
                 except IOError:
@@ -840,7 +929,7 @@ class Logger(object):
         """
         Writes the data string to the raw and hex logs.  The trans_type is
         prepended to each log message and indicates whether this data was
-        just received or is about to be sent.        
+        just received or is about to be sent.
         """
         self.log("hex", trans_type + "\n" + hexdump(data))
         self.log("raw", trans_type + " " + repr(data))
@@ -976,7 +1065,14 @@ class TCPHandler(ProtHandler, StreamRequestHandler):
     """
     Subclass this class for use with the SocketServer.TCPServer class or any
     of its subclasses, such as protlib.LoggingTCPServer
+    
+    This class overrides the rbufsize field of StreamRequestHandler, setting
+    it to 0.  This allows calls to self.rfile.read to not interfere with
+    calls to self.socket.recv, at the cost of being less efficient.
     """
+    rbufsize = 0
+    timeout = None
+    
     def __init__(self, request, client_addr, server):
         """you'll probably never need to instantiate this class directly"""
         ProtHandler.__init__(self, server)
@@ -985,18 +1081,25 @@ class TCPHandler(ProtHandler, StreamRequestHandler):
     def read(self, n=None):
         """
         Repeatedly calls select on self.rfile and returns the next n bytes,
-        or however much data is available by the time the socket times out.
-        If the user has called socket.setdefaulttimeout(), then that value
-        is used, otherwise the global DEFAULT_TIMEOUT constant is used,
-        which is 5 seconds.
+        or however much data is available before a timeout. The value of
+        this handler's self.timeout or self.server.timeout is used to
+        determine the timeout; both are None by default, which indicates
+        an indefinite block.
+        
+        This method returns quickly if this handler's server's socket has
+        been closed.
         """
         buf = ""
-        while True:
-            r,w,exc = select([self.rfile], [], [], socket.getdefaulttimeout() or DEFAULT_TIMEOUT)
-            c = r[0].read(1) if r else ""
-            buf += c                                    # shlemiel the painter
-            if not r or not c or n and len(buf) >= n:
-                break
+        timeout = self.timeout or self.server.timeout
+        wait = min(1, timeout or 1)
+        until = timeout and (time() + timeout)
+        while _is_open(self.server.socket) and (not until or time() < until):
+            r,_,_ = select([self.request], [], [], wait)
+            if r:
+                c = self.request.recv(4096 if n is None else (n - len(buf)))
+                buf += c
+                if not c or n is not None and len(buf) >= n:
+                    break
         return buf
 
 class UDPHandler(ProtHandler, DatagramRequestHandler):
