@@ -1,3 +1,4 @@
+# -*- coding: latin-1 -*-
 import protlib
 from protlib import *
 
@@ -8,19 +9,20 @@ import atexit
 import socket
 import logging
 from glob import glob
-from time import sleep
+from time import sleep, time
 from threading import Thread
 from StringIO import StringIO
-from logging import getLogger, INFO
 
-import unittest
-from unittest import TestCase
+from unittest import TestCase, main
 
 import warnings
 warnings.simplefilter("error", CWarning)
 
-socket.setdefaulttimeout(0.1)
-getLogger().level = INFO
+def assert_works_without_warnings(func, *args, **kwargs):
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", CWarning)
+        func(*args, **kwargs)
+        protlib.__warningregistry__.clear()
 
 def delete_logs():
     atexit._exithandlers[:] = []    # stop the logging module's exit handler
@@ -67,8 +69,8 @@ class TestHandler(object):
         pg.points *= 2
         return pg
 
-class TCPTestHandler(TCPHandler, TestHandler): pass
 class UDPTestHandler(UDPHandler, TestHandler): pass
+class TCPTestHandler(TCPHandler, TestHandler): timeout = 0.1
 
 class CTypeTests(TestCase):
     def test_valid_basic(self):
@@ -79,6 +81,7 @@ class CTypeTests(TestCase):
                               CFloat, CDouble]:
                     ctype(always=always, default=default)
                 CString(length=5, always=always, default=default)
+                CUnicode(length=5, always=always, default=default, encoding="utf8")
     
     def test_invalid_basic(self):
         self.assertRaises(CError, CType)
@@ -154,12 +157,11 @@ class CTypeTests(TestCase):
         
         self.assertRaises(CWarning, CArray(2,CInt).serialize, [5,6,11])
         self.assertRaises(CWarning, CArray(2, CString(length=5)).serialize, ["Hello","World!"])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", CWarning)
-            self.assertEqual(buf, CArray(2,CInt).serialize([5,6,11]))
+        
+        assert_works_without_warnings(lambda: self.assertEqual(buf, CArray(2,CInt).serialize([5,6,11])))
+        assert_works_without_warnings(lambda:
             self.assertEqual(Words(message=["Bye","Cruel"],         target="Error!").serialize(),
-                             Words(message=["Bye","Cruel","World"], target="Error!").serialize())
-            protlib.__warningregistry__.clear()
+                             Words(message=["Bye","Cruel","World"], target="Error!").serialize()))
         
         words = Words(message=["hello","world"], target="fair")
         words.message[1:] = []
@@ -172,12 +174,9 @@ class CTypeTests(TestCase):
         self.assertRaises(CError, CArray, 2, CChar, default=[0,0,0])
         self.assertRaises(CError, CArray, 2, CChar, default=lambda: [0,0,0])
         
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", CWarning)
-            CArray(2, CChar, always=[0,0,0])
-            CArray(2, CChar, default=[0,0,0])
-            CArray(2, CChar, default=lambda: [0,0,0])
-            protlib.__warningregistry__.clear()
+        assert_works_without_warnings(CArray, 2, CChar, always=[0,0,0])
+        assert_works_without_warnings(CArray, 2, CChar, default=[0,0,0])
+        assert_works_without_warnings(CArray, 2, CChar, default=lambda: [0,0,0])
         
         class Point(CStruct):
             xy = CArray(2, CInt, default=[5,6])
@@ -207,15 +206,15 @@ class CTypeTests(TestCase):
         
         class Matrix(CStruct):
             xs = CArray(2, CArray(2, CInt))
+        
         Matrix(xs=[[5,6],[7,8]])
         self.assertRaises(CError, Matrix, xs=[])
+        
         self.assertRaises(CError, Matrix, xs=[[0,0], [1,1], [0,1]])
         self.assertRaises(CError, Matrix, xs=[[0,0,0], [1,1,1]])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            Matrix(xs=[[0,0], [1,1], [0,1]])
-            Matrix(xs=[[0,0,0], [1,1,1]])
-            protlib.__warningregistry__.clear()
+        
+        assert_works_without_warnings(Matrix, xs=[[0,0], [1,1], [0,1]])
+        assert_works_without_warnings(Matrix, xs=[[0,0,0], [1,1,1]])
     
     def test_valid_structs(self):
         np = NamedPoint(x=5, y=6)
@@ -360,13 +359,115 @@ class CTypeTests(TestCase):
         self.assertRaises(CError, CArray, 2, CInt, always=[0, 1.2])
         self.assertRaises(CError, CArray, 2, CInt, default=[0, 1.2])
         self.assertRaises(CError, CArray, 2, CInt, default=lambda:[0, 1.2])
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore", CWarning)
-            NamedPoint(x=5.6)
-            CArray(2, CInt, always=[0, 1.2])
-            CArray(2, CInt, default=[0, 1.2])
-            CArray(2, CInt, default=lambda:[0, 1.2])
-            protlib.__warningregistry__.clear()
+        
+        assert_works_without_warnings(NamedPoint, x=5.6)
+        assert_works_without_warnings(CArray, 2, CInt, always=[0, 1.2])
+        assert_works_without_warnings(CArray, 2, CInt, default=[0, 1.2])
+        assert_works_without_warnings(CArray, 2, CInt, default=lambda:[0, 1.2])
+    
+    def test_invalid_unicode_params(self):
+        self.assertRaises(CError, CInt, encoding = "utf8")
+        self.assertRaises(CError, CInt, enc_errors = "ignore")
+        self.assertRaises(CError, CUnicode)
+        self.assertRaises(CError, CUnicode, length=5)
+        self.assertRaises(CError, CUnicode, encoding="utf8")
+        self.assertRaises(CError, CUnicode, length=5, encoding=None)
+        self.assertRaises(CError, CUnicode, length=5, encoding="invalid encoding")
+    
+    def test_valid_unicode_params(self):
+        CUnicode(length=5, encoding="utf8")
+        CUnicode(length=5, encoding="utf8", enc_errors="strict")
+        CUnicode(length=5, encoding="utf8", enc_errors="ignore")
+        CUnicode(length=5, encoding="utf8", enc_errors="replace")
+        CUnicode(length=5, encoding="utf8", default=u"hello")
+        CUnicode(length=5, encoding="utf8", default="hello")
+    
+    def test_unicode_integrity(self):
+        andre = u"andr"
+        
+        cu = CUnicode(length=6, encoding="latin-1")
+        self.assertEqual(andre, cu.parse(cu.serialize(andre)))
+        
+        class Person(CStruct):
+            name = CUnicode(length=9, encoding="utf8")
+        p = Person(andre)
+        s = Person(andre).serialize()
+        self.assertEqual(p, Person.parse(s))
+        self.assertEqual(andre, p.name)
+        self.assertEqual(p.name, Person.parse(s).name)
+    
+    def test_unicode_coercion(self):
+        andre = u"andr"
+        class Name(CStruct):
+            first = CString(length=9)
+            last = CUnicode(length=9, encoding="utf8")
+        
+        self.assertRaises(CError, Name, first=andre)
+        name = Name(first=u"andre", last="giant")
+        self.assertTrue(isinstance(name.first, str))
+        self.assertTrue(isinstance(name.last, unicode))
+        
+        name = Name(first=5, last=5)
+        self.assertEqual(name.last, "5")
+        self.assertEqual(name.first, "5")
+        self.assertTrue(isinstance(name.first, str))
+        self.assertTrue(isinstance(name.last, unicode))
+        
+        s = CUnicode(length=6, encoding="utf8").serialize(andre)
+        self.assertEqual(andre, Name(last=s).last)
+    
+    def test_encoding_errors(self):
+        class Person(CStruct):
+            name = CUnicode(length=9, encoding="utf8")
+        self.assertRaises(CError, Person, name="\x80")
+        self.assertRaises(CError, setattr, Person(), "name", "\x80")
+        
+        class Person(CStruct):
+            name = CUnicode(length=9, encoding="utf8", enc_errors="ignore")
+        self.assertEqual("", Person(name="\x80").name)
+        p = Person()
+        p.name = "\x80"
+        self.assertEqual("", p.name)
+        
+        class Person(CStruct):
+            name = CUnicode(length=9, encoding="utf8", enc_errors="replace")
+        self.assertEqual(u"\ufffd", Person(name="\x80").name)
+        p = Person()
+        p.name = "\x80"
+        self.assertEqual(u"\ufffd", p.name)
+    
+    def test_length_problems(self):
+        self.assertRaises(CWarning, CUnicode(length=5, encoding="utf8").serialize, u"andr")
+        
+        class Person(CStruct):
+            name = CString(length=5, default="marvin")
+        self.assertRaises(CError, Person)
+        assert_works_without_warnings(Person)
+        
+        class Person(CStruct):
+            name = CString(length=5)
+        p = Person()
+        self.assertRaises(CError, setattr, p, "name", "marvin")
+        assert_works_without_warnings(setattr, p, "name", "marvin")
+        
+        class Person(CStruct):
+            name = CUnicode(length=5, encoding="utf8", default=u"andr")
+        self.assertRaises(CError, Person)
+        assert_works_without_warnings(Person)
+        
+        class Person(CStruct):
+            name = CUnicode(length=5, encoding="utf8")
+        p = Person()
+        self.assertRaises(CError, setattr, p, "name", u"andr")
+        assert_works_without_warnings(setattr, p, "name", u"andr")
+        
+        class Name(CStruct):
+            first = CString(length=5)
+            last = CUnicode(length=5, encoding="utf8")
+        self.assertRaises(CError, Name, first="marvin")
+        self.assertRaises(CError, Name, last=u"andr")
+        assert_works_without_warnings(Name, first="marvin")
+        assert_works_without_warnings(Name, last=u"andr")
 
 class VarlengthTests(TestCase):
     def test_varlength_structs(self):
@@ -515,6 +616,76 @@ class VarlengthTests(TestCase):
             arr = CArray("size", CInt(default=5))
         self.assertEqual(DerivedDefault().arr, [5,5])
 
+class UnsizedTests(TestCase):
+    def test_basic(self):
+        self.assertEqual("hello", CString(length=AUTOSIZED).parse("hello\0world"))
+        self.assertEqual("hello\0", CString(length=AUTOSIZED).serialize("hello"))
+        self.assertEqual("hello\0", CString(length=AUTOSIZED).serialize("hello\0\0\0"))
+    
+    def test_structs(self):
+        buf = "\0\x01asher\0\0\x04"
+        class Person(CStruct):
+            code = CShort(always = 1)
+            name = CString(length = AUTOSIZED)
+            age = CShort()
+        p = Person.parse(buf)
+        self.assertEqual(p.code, 1)
+        self.assertEqual(p.name, "asher")
+        self.assertEqual(p.age, 4)
+        self.assertEqual(buf, p.serialize())
+    
+    def test_nested(self):
+        class Person(CStruct):
+            code = CChar(always = 1)
+            name = CString(length = AUTOSIZED)
+            age = CShort()
+        
+        class Department(CStruct):
+            name = CString(length = AUTOSIZED)
+            people = CArray(2, Person.get_type(default=Person(name="", age=0)))
+        
+        default_person = "\x01\0\0\0"
+        buf = "foo\0" + default_person * 2
+        self.assertEqual(buf, Department("foo").serialize())
+        self.assertEqual(Department("foo"), Department.parse(buf))
+        
+        buf = "foo\0\x01Eli\0\0\x1c\x01Asher\0\0\x04"
+        dept = Department("foo", [Person(name="Eli", age=28), Person(name="Asher", age=4)])
+        self.assertEqual(buf, dept.serialize())
+        self.assertEqual(dept, Department.parse(buf))
+    
+    def test_unicode(self):
+        andre = u"andr"
+        
+        class Person(CStruct):
+            code = CChar(always = 1)
+            name = CUnicode(length=AUTOSIZED, encoding="utf8")
+            age = CShort()
+        
+        class Department(CStruct):
+            name = CUnicode(length=AUTOSIZED, encoding="utf8")
+            people = CArray(2, Person.get_type(default=Person(name=u"", age=0)))
+        
+        default_person = "\x01\0\0\0"
+        buf = "foo\0" + default_person * 2
+        self.assertEqual(buf, Department("foo").serialize())
+        self.assertEqual(Department("foo"), Department.parse(buf))
+        
+        buf = "foo\0\x01andr\xc3\xa9\0\0\x1c\x01Asher\0\0\x04"
+        dept = Department("foo", [Person(name=andre, age=28), Person(name="Asher", age=4)])
+        self.assertEqual(buf, dept.serialize())
+        self.assertEqual(dept, Department.parse(buf))
+    
+    def test_unicode_null_problem(self):
+        s = u"Hello World!".encode("utf-32")
+        self.assertTrue(s.count("\0"))
+        self.assertRaises(CError, CUnicode(length=AUTOSIZED, encoding="utf-32").parse, s)
+        
+        class Person(CStruct):
+            name = CUnicode(length=AUTOSIZED, encoding="utf-32")
+        self.assertRaises(CError, Person.parse, s)
+        
+
 class UnderscorizeTests(TestCase):
     def test_camelcase(self):
         self.assertEqual("some_struct",   underscorize("SomeStruct"))
@@ -551,7 +722,29 @@ class BadHandlerTests(TestCase):
                     code = CInt(always = 1)
         self.assertRaises(CError, BadNameHandler)
 
-class ServerTests:
+class LoggerTests(TestCase):
+    logfiles = ["logger_test.{0}_log".format(log) for log in Logger.LEVELS]
+    
+    def setUp(self):
+        self.delete_logs()
+    
+    def tearDown(self):
+        self.delete_logs()
+    
+    def delete_logs(self):
+        for fname in self.logfiles:
+            if os.path.exists(fname):
+                os.remove(fname)
+    
+    def test_logfile_creation(self):
+        Logger(prefix = "logger_test")
+        for fname in self.logfiles:
+            if sys.version_info[:3] >= (2, 6, 2):
+                self.assertFalse(os.path.exists(fname))
+            else:
+                self.assertTrue(os.path.exists(fname))
+
+class ServerTestBase:
     def setUp(self):
         self.reset_logs()        
         self.server = self.ServerClass(SERVER_ADDR, self.HandlerClass)
@@ -566,11 +759,20 @@ class ServerTests:
     
     def tearDown(self):
         self.sock.close()
+        self.server.server_close()
         self.server.shutdown()
-        self.server.socket.close()
         self.reset_logs()
     
+    def send(self, data):
+        self.f.write( str(data) )
+        self.f.flush()
+    
+    def client_setup(self):
+        self.sock = socket.create_connection(SERVER_ADDR)
+        self.f = self.sock.makefile("r+b", bufsize=0)
+    
     def client_teardown(self):
+        self.f.close()
         self.sock.close()
     
     def reset_logs(self):
@@ -579,10 +781,13 @@ class ServerTests:
             if handlers and handlers[0].stream:
                 handlers[0].stream.truncate(0)
     
+    def reset_logs(self): pass
+    
     def read_log(self, name):
         with open("unit_tests." + name + "_log") as f:
             return f.read()
-    
+
+class ServerTests(ServerTestBase):
     def test_struct_response(self):
         self.send(self.np)
         s = self.f.read( RenamedPoint.sizeof() )
@@ -630,14 +835,6 @@ class ServerTests:
 class TCPServerTests(ServerTests, TestCase):
     ServerClass = LoggingTCPServer
     HandlerClass = TCPTestHandler
-    
-    def client_setup(self):
-        self.sock = socket.create_connection(SERVER_ADDR)
-        self.f = self.sock.makefile("r+b", bufsize=0)
-    
-    def send(self, data):
-        self.f.write( str(data) )
-        self.f.flush()
 
 class UDPServerTests(ServerTests, TestCase):
     ServerClass = LoggingUDPServer
@@ -652,7 +849,37 @@ class UDPServerTests(ServerTests, TestCase):
     def send(self, data):
         self.sock.sendto(str(data), SERVER_ADDR)
 
+class TCPTimeoutTests(ServerTestBase, TestCase):
+    ServerClass = LoggingTCPServer
+    class HandlerClass(TCPHandler):
+        timeout = 2
+    
+    def test_immediate_timeout(self):
+        before = time()
+        self.server.server_close()
+        self.server.shutdown()
+        self.assertTrue(time() - before < 2)
+    
+    def test_full_timeout(self):
+        before = time()
+        self.server.shutdown()
+        self.assertTrue(time() - before > 2)
+
+class TCPReadRecvInteract(ServerTestBase, TestCase):
+    ServerClass = LoggingTCPServer
+    class HandlerClass(TCPHandler):
+        def named_origin(self, no):
+            NamedOrigin.parse(self.rfile)
+            return NamedOrigin()
+    
+    def test_recv_read_interaction(self):
+        for i in range(9):
+            self.send(NamedOrigin())
+            self.send(NamedOrigin())
+            NamedOrigin.parse(self.f)
+
 if __name__ == "__main__":
+    logging.basicConfig(level = logging.INFO)
     delete_logs()
     atexit.register(delete_logs)
-    unittest.main()
+    main()
